// -*- C++ -*-
//
// ChBenaphore - smarter/faster semaphore
//
// Copyright © 1996-1998 Arcane Dragon Software
//                       Chris Herborth (chrish@kagi.com)
//
// ChBenaphore is a class implementing the "benaphore" idea that Benoit
// Schillings discussed in the Be Newsletter, Issue #26.
//
// A benaphore is basically a "smart" semaphore that tries to avoid the
// overhead of aquire_sem() and release_sem() calls by keeping an atomic
// counter handy.
//
// $Id: ChBenaphore.h,v 1.5 1998/04/14 11:39:23 chrish Exp $

#ifndef libCh_BENAPHORE_H
#define libCh_BENAPHORE_H

#pragma once

#include <kernel/OS.h>
#include <support/SupportDefs.h>

class ChBenaphore
{
public:
	ChBenaphore( const char *name = "ChBenaphore" );
	virtual ~ChBenaphore( void );

	bool IsValid( void ) {
		return ( _ben_sem < B_NO_ERROR ? false : true );
	}

	status_t Lock( void );
	status_t LockTimeout( bigtime_t microseconds = 0 );

	void Unlock( void );

private:
	ChBenaphore( const ChBenaphore& );		// Don't copy me
	ChBenaphore& operator=( ChBenaphore& );	// Don't assign me

	sem_id _ben_sem;
	int32  _atom;

	static char *_rcs_id;
};

#endif
