// -*- C++ -*-
//
// ChPulsar - send a message periodically
//
// Copyright © 1998 Arcane Dragon Software
//                  Chris Herborth (chrish@kagi.com)
//
// This is similar to the Pulse() method (and B_PULSE message) found in
// BView.  Pulse() won't be called if the view is responding to any other
// messages... thus, it's not reliable.  ChPulsar spawns a separate thread,
// and will be reliable +/- some scheduling latency.
//
// $Id: ChPulsar.h,v 1.4 1998/04/14 11:39:23 chrish Exp $

#ifndef libCh_PULSAR_H
#define libCh_PULSAR_H

#pragma once

#include <kernel/OS.h>
#include <app/Message.h>
#include <app/Handler.h>
#include <app/Messenger.h>

#include "Ch/ChSnoozer.h"

class ChPulsar
{
public:
	ChPulsar( const bigtime_t interval, BMessage *message = NULL,
			  const char *name = "ChPulsar", BHandler *handler = NULL,
			  int32 priority = B_URGENT_PRIORITY );
	virtual ~ChPulsar( void );

	bool IsValid( void ) {
		if( _pulsar_tid >= B_NO_ERROR && 
			_snoozer->IsValid()       && 
			_target->IsValid()         ) {
			return true;
		} else {
			return false;
		}
	}

	status_t Start( void );
	status_t Suspend( void );
	
	status_t Status( thread_state *state );
	bigtime_t GetInterval( void ) { return _interval; }
	void SetInterval( bigtime_t new_interval ) { _interval = new_interval; }

private:
	ChPulsar( const ChPulsar& );		// Don't copy me
	ChPulsar& operator=( ChPulsar& );	// Don't assign me

	thread_id _pulsar_tid;
	ChSnoozer *_snoozer;
	bigtime_t _interval;
	BMessage *_msg;
	BMessenger *_target;

	uint64 _seq;

	static int32 _pulsar_thread( void *data );
	bool _exit_pulsar;

	static char *_rcs_id;
};

#endif
